/*##########################################################################
#                                                                          #
#                        rnaapp.exe patcher  v1.0                          #
#                                                                          #
#              Copyright (c) 2001 SOLEIL VERT / David TAILLANDIER          #
#                                                                          #
############################################################################

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

############################################################################

The full program is into this file.

Revision history :

december 22st 2001 v1.2  David TAILLANDIER / SOLEIL VERT, Initial release.

############################################################################

This utility patches c:\windows\system\rnaapp.exe witch is the remote access
server for Microsoft Windows 98 (and Win95 with an update).
note: this program have been tested only with rnaapp.exe from Win98. It is
strongly possible that the file from Win95 can be patched the same way, but
not 100% sure.

This utility *may* patches rnaapp.exe for every language, but we have only
tested with french.

The patch is intended to let rnaapp.exe ignore WM_QUESRYENDSESSION and
WM_ENDSESSION messages when user logoff (but not when user shut down the
computer). This is usefull when you register rnaapp.exe as a service because
you can then reach the computer when the user is logged on or off, without
getting disconnected. The computer can then acts as an ISDN router for IP
protocol, permitting file transfer, remote access, etc.

The patch does not resolve the problem occuring when the computer goes into
sleep mode. If you are connected, a message box appear. If you are not
connected, the computer goes into sleep mode and no connection will be
possible. But, hey, an ISDN router is not allowed to sleep !

To register rnaapp.exe as a service, you need another utility. You can use
winservice.exe for example.
This path utility did not include service registration inside rnaapp.exe
because of a buggy implementation of a service program inside kernel32.dll


###########################################################################*/

#include <stdlib.h>
#include <stdio.h>
#include <mem.h>

void error( const char * text );
void patch( char * buffer, unsigned int buflen, const char * matchdata, unsigned int matchlen, const char * patchdata, unsigned int patchlen, int patchoffset );

/*############################### main ####################################*/
int
main( int argc, char * argv[] )
{
    char   match1[] = {0x83,0x7c,0x24,0x1c,0x00,0x75,0x0a,0xe8,0xa4,0xfc,
                       0xff, 0xff, 0xe9, 0xf8, 0xfe, 0xff, 0xff, 0x33, 0xc0,
                       0xe9, 0xf1, 0xfe, 0xff, 0xff};
    char   patch1[]  = {0xb8,0x01,0x00,0x00,0x00,0xe9,0xff,0xfe,0xff,0xff};
    int    patch1offset = 0;

    char   match2[] = {0x83,0x7c,0x24,0x1c,0x00,0x74,0x0c,0xe8,0xfe,0xfd,
                       0xff,0xff,0x33,0xc0,0xe9,0xde,0xfe,0xff,0xff};
    char   patch2[]  = {0x83,0x7c,0x24,0x20,0x00,0x75,0x05};
    int    patch2offset = 0;

    char * buffer;
    FILE * handle;
    unsigned int filesize;

    printf( "rnaapp.exe patch v1.0       Copyright (c) 2001 SOLEIL VERT / David TAILLANDIER\n"
            "\n"
            "This program is free software. You should have received a copy of the GNU\n"
            "General Public License along with this program; if not, write to the Free\n"
            "Software Foundation, Inc., s59 Temple Place - Suite 330, Boston,\n"
            "MA  02111-1307, USA.\n"
            "\n"
            "Using command line: rnpatch <filename>\n"
            "Using Windows Explorer: you can drag-and-drop the file over rnpatch.exe\n"
            "\n"
         );

    /* allocate memory for buffer */
    buffer = (char *)malloc( 60*1024 );  /* 60kb may be ok for every languages */
    if ( buffer == NULL )
       error( "Unable to allocate 60kb. Maybe the program is compiled with a wrong memory\n"
              "model (you can't use tiny if you compile in 16-bit mode\n" );

    /* check command-line */
    if ( argc <= 1 )
        error( "You must submit a filename\n" );
    if ( argc > 2 )
        error( "Only one filename can be submited. Don't use spaces nor long filename,\n"
                "this is a very basic MS-DOS program\n" );

    /* read the entire file into memory */
    handle = fopen( argv[1], "r+b" );
    if ( handle == NULL )
        error( "Unable to open target file.\n" );
    rewind( handle );
    filesize = (unsigned int)fread( buffer, 1, 60*1024, handle );
    if ( filesize<(30U*1024U) || filesize>(59U*1024U) )
       error( "This file have not the right size to be rnaapp.exe\n" );

    /* patch */
    patch( buffer, filesize, match1, sizeof(match1), patch1, sizeof(patch1), patch1offset );
    patch( buffer, filesize, match2, sizeof(match2), patch2, sizeof(patch2), patch2offset );

    /* write to file */
    rewind( handle );

    if ( fwrite(buffer,1,filesize,handle) != filesize )
       error( "Write error into file.\n" );
  
    printf( "The file is now patched.\n" );

    fclose( handle );   /* we don't check return value (hooo!) */
    return 0;
}

/*############################ patch ######################################*/
void
patch( char * buffer,
       unsigned int buflen,
       const char * matchdata,
       unsigned int matchlen,
       const char * patchdata,
       unsigned int patchlen,
       int patchoffset )
{
    int matchcount = 0;
    char * bufptr = buffer;

    while( 1 )    {
        bufptr = (char *)memchr( (const void *)bufptr, matchdata[0], buflen-matchlen-(bufptr-buffer) );
        if ( bufptr == NULL )
            break;
        if ( memcmp(bufptr,matchdata,matchlen) == 0 )    {
            memcpy( bufptr+patchoffset, patchdata, patchlen );
            matchcount++;
            if ( matchcount == 2 )
                break;
        }
        bufptr++;
    }

    if ( matchcount != 1 )
        error( "This file does not contain code to patch. Maybe this is not a rnaapp.exe\n" );

}

/*############################ error ######################################*/
void
error( const char * text )
{
    printf( text );
    exit( 0 );
}

/*########################## END OF C SOURCE ##############################*/

/*  here is the 'source' code of rnaapp.exe in the area dealing with end of
session. The interesting things are WindowProc and the last lines.

ByeBye1:
:004011DF 6A00           push   00000000                   
:004011E1 6A00           push   00000000                   
:004011E3 6A3D           push   0000003D                   
:004011E5 E878380000     call   RASAPI32.SuprvRequest      
:004011EA 85C0           test   eax,eax                    
:004011EC B801000000     mov    eax,00000001               
:004011F1 7426           je     RNAAPP.00401219 (00401219)   --> return
:004011F3 6A00           push   00000000                   
:004011F5 A194534000     mov    eax,[00405394]             
:004011FA 681A124000     push   0040121A                lpDialogFunc   
:004011FF 50             push   eax                        
:00401200 681A040000     push   0000041A                template #1050   
:00401205 FF3570524000   push   dword ptr [00405270]    
:0040120B FF1588624000   call   [00406288]              display dialog box
:00401211 48             dec    eax                        
:00401212 83F801         cmp    eax,00000001               
:00401215 1BC0           sbb    eax,eax                    
:00401217 F7D8           neg    eax                        
:00401219 C3             ret                               

lpDialogFunc:
:0040121A 55             push   ebp                        
:0040121B 8B44240C       mov    eax,[esp+0C]               
:0040121F 8BEC           mov    ebp,esp                    
:00401221 83F802         cmp    eax,00000002                    WM_DESTROY
:00401224 7417           je     RNAAPP PC DAVID.0040123D
:00401226 3D10010000     cmp    eax,00000110                    WM_INITDIALOG
:0040122B 7427           je     RNAAPP PC DAVID.00401254
:0040122D 3D11010000     cmp    eax,00000111                    WM_COMMAND
:00401232 7445           je     RNAAPP PC DAVID.00401279
:00401234 3D13010000     cmp    eax,00000113                    WM_TIMER
:00401239 744F           je     RNAAPP PC DAVID.0040128A
:0040123B EB58           jmp    RNAAPP PC DAVID.00401295     return 0

WM_DESTROY:
:0040123D 6A08           push   00000008                   
:0040123F FF7508         push   dword ptr [ebp+08]         
:00401242 FF15A0624000   call   [004062A0]                 ? timer's handle ?
:00401248 50             push   eax                        
:00401249 FF7508         push   dword ptr [ebp+08]         
:0040124C FF1554634000   call   [00406354]                 ? kill timer ?
:00401252 EB41           jmp    RNAAPP PC DAVID.00401295      return 0

WM_INITDIALOG:
:00401254 6A00           push   00000000                   no callback
:00401256 68983A0000     push   00003A98                   15 seconds
:0040125B 6A02           push   00000002                   timer identifier = 2
:0040125D FF7508         push   dword ptr [ebp+08]         
:00401260 FF1550634000   call   [00406350]                 SetTimer
:00401266 50             push   eax                        
:00401267 6A08           push   00000008                   
:00401269 FF7508         push   dword ptr [ebp+08]         
:0040126C FF1590624000   call   [00406290]                 ? save timer's handle ?
:00401272 B801000000     mov    eax,00000001               
:00401277 EB1E           jmp    RNAAPP PC DAVID.00401297    return 1


WM_COMMAND:
:00401279 0FB74510       movzx  eax,word ptr [ebp+10]      
:0040127D 83F801         cmp    eax,00000001               
:00401280 7C13           jl     RNAAPP PC DAVID.00401295    return 0
:00401282 83F802         cmp    eax,00000002               
:00401285 7F0E           jg     RNAAPP PC DAVID.00401295    return 0
:00401287 50             push   eax                         buton "ok" or "cancel"
:00401288 EB02           jmp    RNAAPP PC DAVID.0040128C

WM_TIMER:
:0040128A 6A01           push   00000001                   disconnection (like buton "cancel" ?)
:0040128C FF7508         push   dword ptr [ebp+08]         
:0040128F FF158C624000   call   [0040628C]                 EndDialog

return 0:
:00401295 33C0           xor    eax,eax                    

return:
:00401297 5D             pop    ebp                        
:00401298 C21000         ret    0010                    

   
:0040129B 833D2057400000 cmp    dword ptr [00405720],000000
:004012A2 B801000000     mov    eax,00000001               
:004012A7 7426           je     RNAAPP PC DAVID.004012CF (0
:004012A9 6A00           push   00000000                   
:004012AB A194534000     mov    eax,[00405394]             
:004012B0 68D0124000     push   004012D0                   
:004012B5 50             push   eax                        
:004012B6 681B040000     push   0000041B                   
:004012BB FF3570524000   push   dword ptr [00405270]       
:004012C1 FF1588624000   call   [00406288]                 
:004012C7 48             dec    eax                        
:004012C8 83F801         cmp    eax,00000001               
:004012CB 1BC0           sbb    eax,eax                    
:004012CD F7D8           neg    eax                        


ByeBye2:
:00401351 83EC1C         sub    esp,0000001C               
:00401354 833D2057400000 cmp    dword ptr [00405720],000000
:0040135B 53             push   ebx                        
:0040135C 56             push   esi                        
:0040135D 57             push   edi                        
:0040135E 55             push   ebp                        
:0040135F 0F8487000000   je     RNAAPP.004013EC            
:00401365 C705B853400001+mov    dword ptr [004053B8],000000
:0040136F 6A00           push   00000000                   
:00401371 6A00           push   00000000                   
:00401373 6A6A           push   0000006A                   
:00401375 E8EE360000     call   RASAPI32.DialEngineRequest 
:0040137A 6A00           push   00000000                   
:0040137C 6A00           push   00000000                   
:0040137E 6A02           push   00000002                   
:00401380 E8DD360000     call   RASAPI32.SuprvRequest      
:00401385 8B3D00544000   mov    edi,[00405400]             
:0040138B 85FF           test   edi,edi                    
:0040138D 741A           je     RNAAPP.004013A9 (004013A9) 
:0040138F 8B359C624000   mov    esi,[0040629C]         

loop:    
:00401395 6A03           push   00000003                   
:00401397 6AFF           push   FFFFFFFF                   
:00401399 68CDCC0000     push   0000CCCD                   
:0040139E FF37           push   dword ptr [edi]            
:004013A0 FFD6           call   esi                        
:004013A2 8B7F08         mov    edi,[edi+08]               
:004013A5 85FF           test   edi,edi                    
:004013A7 75EC           jne    RNAAPP.00401395 (00401395)    --> loop

:004013A9 833DB853400000 cmp    dword ptr [004053B8],000000
:004013B0 7443           je     RNAAPP.004013F5 (004013F5) --> return
:004013B2 8B35AC624000   mov    esi,[004062AC]             
:004013B8 8B3DA8624000   mov    edi,[004062A8]             
:004013BE 8B1DA4624000   mov    ebx,[004062A4]             
:004013C4 33ED           xor    ebp,ebp                    
:004013C6 8D442410       lea    eax,[esp+10]               
:004013CA 55             push   ebp                        
:004013CB 55             push   ebp                        
:004013CC 55             push   ebp                        
:004013CD 50             push   eax                        
:004013CE FFD3           call   ebx                        
:004013D0 85C0           test   eax,eax                    
:004013D2 7421           je     RNAAPP.004013F5 (004013F5)  --> return
:004013D4 8D442410       lea    eax,[esp+10]               
:004013D8 50             push   eax                        
:004013D9 FFD6           call   esi                        
:004013DB 8D442410       lea    eax,[esp+10]               
:004013DF 50             push   eax                        
:004013E0 FFD7           call   edi                        
:004013E2 392DB8534000   cmp    [004053B8],ebp             
:004013E8 75DC           jne    RNAAPP.004013C6 (004013C6) 
:004013EA EB09           jmp    RNAAPP.004013F5 (004013F5) 

:004013EC 6A00           push   00000000                   
:004013EE 6A02           push   00000002                   
:004013F0 E867360000     call   RASAPI32.RnaEngineRequest  

return:
:004013F5 5D             pop    ebp                        
:004013F6 5F             pop    edi                        
:004013F7 5E             pop    esi                        
:004013F8 5B             pop    ebx                        
:004013F9 83C41C         add    esp,0000001C               
:004013FC C3             ret                               

WindowProc:
:004013FD 53             push   ebx                        
:004013FE 56             push   esi                        
:004013FF 8B742410       mov    esi,[esp+10]               
:00401403 57             push   edi                        
:00401404 83FE10         cmp    esi,00000010               WM_CLOSE
:00401407 55             push   ebp                        
:00401408 8BC6           mov    eax,esi                    
:0040140A 7733           ja     RNAAPP.0040143F (0040143F)       --> interesting-jump
:0040140C 0F8409010000   je     RNAAPP.0040151B            
:00401412 83F801         cmp    eax,00000001               WM_CREATE
:00401415 0F848C000000   je     RNAAPP.004014A7            
:0040141B 83F802         cmp    eax,00000002               WM_DESTROY
:0040141E 0F84B2000000   je     RNAAPP.004014D6            

DefWinProc:
:00401424 8B7C2420       mov    edi,[esp+20]               
:00401428 57             push   edi                        
:00401429 FF742420       push   dword ptr [esp+20]         
:0040142D 56             push   esi                        
:0040142E FF742420       push   dword ptr [esp+20]         
:00401432 FF15B8624000   call   [004062B8]                 

return:
:00401438 5D             pop    ebp                        
:00401439 5F             pop    edi                        
:0040143A 5E             pop    esi                        
:0040143B 5B             pop    ebx                        
:0040143C C21000         ret    0010                       


interesting-jump:
:0040143F 83F816         cmp    eax,00000016               
:00401442 7711           ja     RNAAPP.00401455 (00401455) 
:00401444 0F84FD000000   je     RNAAPP.00401547               --> endsession             
:0040144A 83F811         cmp    eax,00000011                                             
:0040144D 0F84DC000000   je     RNAAPP.0040152F               --> queryendsession        
:00401453 EBCF           jmp    RNAAPP.00401424 (00401424)    --> DefWinProc

:00401455 3D18020000     cmp    eax,00000218                 WM_POWERBROADCAST
:0040145A 7713           ja     RNAAPP.0040146F (0040146F) 
:0040145C 0F84A0010000   je     RNAAPP.00401602            
:00401462 3D13010000     cmp    eax,00000113                 WM_TIMER
:00401467 0F84F7000000   je     RNAAPP.00401564            
:0040146D EBB5           jmp    RNAAPP.00401424 (00401424) 
:0040146F 3D0A040000     cmp    eax,0000040A                 ?
:00401474 7713           ja     RNAAPP.00401489 (00401489) 
:00401476 0F84FE010000   je     RNAAPP.0040167A            
:0040147C 3D19020000     cmp    eax,00000219                 WM_DEVICECHANGE
:00401481 0F848C040000   je     RNAAPP.00401913            
:00401487 EB9B           jmp    RNAAPP.00401424 (00401424) 
:00401489 3DCDCC0000     cmp    eax,0000CCCD                 ?
:0040148E 7794           ja     RNAAPP.00401424 (00401424) 
:00401490 0F84EA030000   je     RNAAPP.00401880            
:00401496 2D0B040000     sub    eax,0000040B               
:0040149B 83F808         cmp    eax,00000008               
:0040149E 7784           ja     RNAAPP.00401424 (00401424) 
:004014A0 FF248545194000 jmp    [4*eax+00401945]      

WM_CREAT:     
:004014A7 FF742414       push   dword ptr [esp+14]         
:004014AB 6A01           push   00000001                   
:004014AD E8AA350000     call   RASAPI32.RnaEngineRequest  
:004014B2 85C0           test   eax,eax                    
:004014B4 750A           jne    RNAAPP.004014C0 (004014C0) 
:004014B6 B8FFFFFFFF     mov    eax,FFFFFFFF               
:004014BB E978FFFFFF     jmp    RNAAPP.00401438            
:004014C0 33C0           xor    eax,eax                    
:004014C2 A3D4534000     mov    [004053D4],eax             
:004014C7 A300544000     mov    [00405400],eax             
:004014CC A304544000     mov    [00405404],eax             
:004014D1 E9A3030000     jmp    RNAAPP.00401879            

WM_DESTROY:
:004014D6 833D2857400000 cmp    dword ptr [00405728],000000
:004014DD 7414           je     RNAAPP.004014F3 (004014F3) 
:004014DF A128574000     mov    eax,[00405728]             
:004014E4 8B3554634000   mov    esi,[00406354]             
:004014EA 50             push   eax                        
:004014EB FF742418       push   dword ptr [esp+18]         
:004014EF FFD6           call   esi                        
:004014F1 EB06           jmp    RNAAPP.004014F9 (004014F9) 

:004014F3 8B3554634000   mov    esi,[00406354]             

:004014F9 833D2C57400000 cmp    dword ptr [0040572C],000000
:00401500 740C           je     RNAAPP.0040150E (0040150E) 
:00401502 A12C574000     mov    eax,[0040572C]             
:00401507 50             push   eax                        
:00401508 FF742418       push   dword ptr [esp+18]         
:0040150C FFD6           call   esi        
                
:0040150E 6A00           push   00000000                   
:00401510 FF15B4624000   call   [004062B4]                 
:00401516 E95E030000     jmp    RNAAPP.00401879            

WM_CLOSE:
:0040151B E831FEFFFF     call   RNAAPP.00401351            
:00401520 FF742414       push   dword ptr [esp+14]         
:00401524 FF1598624000   call   [00406298]                 
:0040152A E94A030000     jmp    RNAAPP.00401879            

queryendsession:                                                         PATCH HERE
:0040152F 837C241C00     cmp    dword ptr [esp+1C],00000000    wParam    mov eax,1    accept end of session B8 01 00 00 00
:00401534 750A           jne    RNAAPP.00401540 (00401540)               jmp 401438                         E9 FF FE FF FF
:00401536 E8A4FCFFFF     call   RNAAPP.004011DF            --> ByeBye1      
:0040153B E9F8FEFFFF     jmp    RNAAPP.00401438            --> return
:00401540 33C0           xor    eax,eax                    refuse end of session
:00401542 E9F1FEFFFF     jmp    RNAAPP.00401438            --> return

endsession:                                                              PATCH HERE
:00401547 837C241C00     cmp    dword ptr [esp+1C],00000000     wParam   cmp dword [esp+20],0  lParam    83 7C 24 20 00
:0040154C 740C           je     RNAAPP.0040155A (0040155A)               jne 401553                      75 05
:0040154E E8FEFDFFFF     call   RNAAPP.00401351            --> ByeBye2
:00401553 33C0           xor    eax,eax                    message processed
:00401555 E9DEFEFFFF     jmp    RNAAPP.00401438            --> return
:0040155A B801000000     mov    eax,00000001               message not processed
:0040155F E9D4FEFFFF     jmp    RNAAPP.00401438            --> return

*/
